#include <windows.h>            // required for all Windows applications
#include <windowsx.h>
#include <shlobj.h>
#include <shellapi.h>
#include "globals.h"            // prototypes specific to this application
#include "resource.h"

HINSTANCE hInst;                // current instance
HWND      g_hwndHidden;

char szAppName[9];              // The name of this application
char szTitle[40];               // The title bar text

//
//  FUNCTION: CenterWindow(HWND, HWND)
//
//  PURPOSE:  Center one window over another.
//
//  PARAMETERS:
//    hwndChild - The handle of the window to be centered.
//    hwndParent- The handle of the window to center on.
//
//  RETURN VALUE:
//
//    TRUE  - Success
//    FALSE - Failure
//
//  COMMENTS:
//
//    Dialog boxes take on the screen position that they were designed
//    at, which is not always appropriate. Centering the dialog over a
//    particular window usually results in a better position.
//

BOOL CenterWindow(HWND hwndChild, HWND hwndParent)
{
    RECT    rcChild, rcParent;
    int     cxChild, cyChild, cxParent, cyParent;
    int     cxScreen, cyScreen, xNew, yNew;
    HDC     hdc;

    // Get the Height and Width of the child window
    GetWindowRect(hwndChild, &rcChild);
    cxChild = rcChild.right - rcChild.left;
    cyChild = rcChild.bottom - rcChild.top;

    // Get the Height and Width of the parent window
    GetWindowRect(hwndParent, &rcParent);
    cxParent = rcParent.right - rcParent.left;
    cyParent = rcParent.bottom - rcParent.top;

    // Get the display limits
    hdc = GetDC(hwndChild);
    cxScreen = GetDeviceCaps(hdc, HORZRES);
    cyScreen = GetDeviceCaps(hdc, VERTRES);
    ReleaseDC(hwndChild, hdc);

    // Calculate new X position, then adjust for screen
    xNew = rcParent.left + ((cxParent - cxChild) / 2);
    if (xNew < 0)
    {
        xNew = 0;
    }
    else if ((xNew + cxChild) > cxScreen)
    {
        xNew = cxScreen - cxChild;
    }

    // Calculate new Y position, then adjust for screen
    yNew = rcParent.top  + ((cyParent - cyChild) / 2);
    if (yNew < 0)
    {
        yNew = 0;
    }
    else if ((yNew + cyChild) > cyScreen)
    {
        yNew = cyScreen - cyChild;
    }

    // Set it, and return
    return SetWindowPos(hwndChild,
                        NULL,
                        xNew, yNew,
                        0, 0,
                        SWP_NOSIZE | SWP_NOZORDER);
}
//
//  FUNCTION: InitApplication(HINSTANCE, int)
//
//  PURPOSE: Initializes window data and registers window class.
//
//  PARAMETERS:
//    hInstance - The handle to the instance of this application that
//                is currently being executed.
//    nCmdShow  - Specifies how the main window is to be displayed.
//
//  RETURN VALUE:
//    TRUE  - Success
//    FALSE - Initialization failed
//
//  COMMENTS:
//
//    This function is called at application initialization time.  It
//    performs initialization tasks for the current application instance.
//    Unlike Win16, in Win32, each instance of an application must register
//    window classes.
//
//    In this function, we initialize a window class by filling out a data
//    structure of type WNDCLASS and calling the Windows RegisterClass()
//    function.  Then we create the main window and show it.
//
//

BOOL InitApplication(HINSTANCE hInstance, int nCmdShow)
{
    WNDCLASSEX wc;
    HWND       hwnd; // Main window handle.

    // Load the application name and description strings.

    LoadString(hInstance, IDS_APPNAME, szAppName, sizeof(szAppName));
    LoadString(hInstance, IDS_DESCRIPTION, szTitle, sizeof(szTitle));

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.

    hInst = hInstance; // Store instance handle in our global variable

    // Fill in window class structure with parameters that describe the
    // main window.

    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = CS_HREDRAW | CS_VREDRAW; // Class style(s).
    wc.lpfnWndProc   = (WNDPROC)WndProc;        // Window Procedure
    wc.cbClsExtra    = 0;                       // No per-class extra data.
    wc.cbWndExtra    = 0;                       // No per-window extra data.
    wc.hInstance     = hInstance;               // Owner of this class
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_APPICON)); // Icon name from .RC
    wc.hCursor       = LoadCursor(hInstance, MAKEINTRESOURCE(IDC_SPLITTER)); // Cursor
    wc.hbrBackground = (HBRUSH)(COLOR_3DFACE + 1); // Default color
    wc.lpszMenuName  = szAppName;               // Menu name from .RC
    wc.lpszClassName = szAppName;               // Name to register as
	wc.hIconSm       = LoadImage(hInstance,		// Load small icon image
	                             MAKEINTRESOURCE(IDI_APPICON),
	                             IMAGE_ICON,
	                             16, 16,
	                             0);

    // Register the window class and return FALSE if unsuccesful.

    if (!RegisterClassEx(&wc))
    {
        //Assume we are running on NT where RegisterClassEx() is
        //not implemented, so let's try calling RegisterClass().

        if (!RegisterClass((LPWNDCLASS)&wc.style))
        	return FALSE;
    }

    // Create a main window for this application instance.
    hwnd = CreateWindow(szAppName,           // See RegisterClass() call
                        szTitle,             // Text for window title bar
                        WS_OVERLAPPEDWINDOW, // Window style
                        90, 90,				 // x, y
                        400, 600,			 // cx, cy
                        NULL,                // Overlapped has no parent
                        NULL,                // Use the window class menu
                        hInstance,           // This instance owns this window
                        NULL                 // Don't need data in WM_CREATE
    );

    // If window could not be created, return "failure"
    if (!hwnd)
        return FALSE;

    // Make the window visible; update its client area; and return "success"
    ShowWindow(hwnd, nCmdShow);  // Show the window
    UpdateWindow(hwnd);          // Sends WM_PAINT message

    return TRUE;                 // We succeeded...
}

//
//  FUNCTION: WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
//
//  PURPOSE: calls initialization function, processes message loop
//
//  PARAMETERS:
//
//    hInstance - The handle to the instance of this application that
//          is currently being executed.
//
//    hPrevInstance - This parameter is always NULL in Win32
//          applications.
//
//    lpCmdLine - A pointer to a null terminated string specifying the
//          command line of the application.
//
//    nCmdShow - Specifies how the main window is to be diplayed.
//
//  RETURN VALUE:
//    If the function terminates before entering the message loop,
//    return FALSE.
//    Otherwise, return the WPARAM value sent by the WM_QUIT message.
//
//
//  COMMENTS:
//
//    Windows recognizes this function by name as the initial entry point
//    for the program.  This function calls the initialization routine
//    It then executes a message retrieval and dispatch loop that is the
//    top-level control structure for the remainder of execution.  The
//    loop is terminated when a WM_QUIT  message is received, at which
//    time this function exits the application instance by returning the
//    value passed by PostQuitMessage().
//
//    If this function must abort before entering the message loop, it
//    returns the conventional value NULL.
//

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance, 
                     LPSTR     lpCmdLine, 
                     int       nCmdShow)
{
    MSG msg;
    HANDLE hAccelTable;

    // Initialize application by setting up the main window.
    if (!InitApplication(hInstance, nCmdShow))
    {
        return FALSE;           // Exits if unable to initialize
    }

    hAccelTable = LoadAccelerators(hInstance, szAppName);

    if (NOERROR != OleInitialize(NULL))
        return FALSE; 

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg, NULL, 0, 0))
    {
        if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg))
        {
            TranslateMessage(&msg);  // Translates virtual key codes
            DispatchMessage(&msg);   // Dispatches message to window
        }
    }

    OleUninitialize();

    return msg.wParam;  // Returns the value from PostQuitMessage
}


#include <commctrl.h>
#include "listview.h"           // prototypes and defines specific to the
                                // listview control

void Error( char* szMessage, ... )
{
  va_list  ap;
	char     szBuffer[ 256 ];

  va_start( ap, szMessage );
  vsprintf( szBuffer, szMessage, ap );

	MessageBox( NULL, szBuffer, "Error", MB_OK | MB_ICONINFORMATION );

  va_end( ap );
}

//Function Prototypes

BOOL PopulateListView(HWND hwndLV,
                      LPTVITEMDATA lptvid,
                      LPSHELLFOLDER lpsf);

extern BOOL GetName(LPSHELLFOLDER lpsf,
                    LPITEMIDLIST  lpi,
			        DWORD         dwFlags,
                    LPSTR         lpFriendlyName);

extern int GetIcon(LPITEMIDLIST lpi, UINT uFlags);

extern LPITEMIDLIST ConcatPidls(LPCITEMIDLIST pidl1, LPCITEMIDLIST pidl2);

extern LPITEMIDLIST CopyITEMID(LPMALLOC lpMalloc, LPITEMIDLIST lpi);

// Module specific globals

HWND hwndLV;
extern HIMAGELIST hImageList;

//
//  FUNCTION: InitListViewImageLists(HWND hwndLV)
//
//
//  PURPOSE: Associate the system image list with our listview.
//
//  PARAMETERS:
//
//    HWND - Handle to the listview control
//
//  RETURN VALUE:
//
//    TRUE on success, FALSE on failure
//
//  COMMENTS:
//

BOOL WINAPI InitListViewImageLists(HWND hwndLV)
{
    HIMAGELIST himlSmall;
    HIMAGELIST himlLarge;
    SHFILEINFO sfi;
    BOOL       bSuccess=TRUE;

	  himlSmall = (HIMAGELIST)SHGetFileInfo((LPCSTR)"C:\\",
                                           0,
                                           &sfi,
                                           sizeof(SHFILEINFO),
                                           SHGFI_SYSICONINDEX | SHGFI_SMALLICON);

    himlLarge = (HIMAGELIST)SHGetFileInfo((LPCSTR)"C:\\",
                                           0,
                                           &sfi,
                                           sizeof(SHFILEINFO),
                                           SHGFI_SYSICONINDEX | SHGFI_LARGEICON);

    if (himlSmall && himlLarge)
    {
       ListView_SetImageList(hwndLV, himlSmall, LVSIL_SMALL);
       ListView_SetImageList(hwndLV, himlLarge, LVSIL_NORMAL);
    }
    else
       bSuccess = FALSE;

    return bSuccess;
}


//
//  FUNCTION: InitListViewItems(HWND hwndLV,
//                              LPTVITEMDATA lptvid,
//                              LPSHELLFOLDER lpsf,
//                              LPITEMIDLIST lpi)
//
//
//  PURPOSE: Add the items to the listview control
//
//  PARAMETERS:
//
//    HWND - Handle to the listview control
//    LPTVITEMDATA - Pointer to TreeView item data
//    LPSHELLFOLDER - Pointer to parent shell folder
//
//  RETURN VALUE:
//
//    TRUE  - if items were successfully added
//    FALSE - if items were not successfully added
//
//  COMMENTS:
//

BOOL WINAPI InitListViewItems(HWND hwndLV,
                              LPTVITEMDATA lptvid,
                              LPSHELLFOLDER lpsf)
{
		ULONG        ulFetched, ulAttrs;
    LPENUMIDLIST lpe=0;
    LV_ITEM      lvi;
    int          iCtr;
    HRESULT      hr;
    LPMALLOC     lpMalloc;
    LPITEMIDLIST lpifqThisItem;
    LPITEMIDLIST lpi=0;
    LPLVITEMDATA lplvid;
    HWND         hwnd=GetParent(hwndLV);
    UINT         uFlags;

    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;

    hr=SHGetMalloc(&lpMalloc);
    if (FAILED(hr))
       return FALSE;

    // Hourglass on!

    SetCapture( GetParent( hwndLV ) );
    SetCursor( LoadCursor( NULL, IDC_WAIT ) );

    if ( SUCCEEDED( hr ) )
    {
        hr = lpsf->lpVtbl->EnumObjects( lpsf, hwnd, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &lpe );

        if ( SUCCEEDED( hr ) )
        {
					BOOL bContinue = TRUE;
            iCtr = 0;

            while ( bContinue )
            {
								bContinue = (lpe->lpVtbl->Next( lpe, 1, &lpi, &ulFetched ) == S_OK );
								if ( !bContinue )
									continue;

                //OK, let's get some memory for our ITEMDATA struct

                lplvid = (LPLVITEMDATA)lpMalloc->lpVtbl->Alloc(lpMalloc, sizeof(LVITEMDATA));
                if (!lplvid)
                   goto Done;

                //Note that since we are interested in the display attributes as well as
                //the other attributes, we need to set ulAttrs to SFGAO_DISPLAYATTRMASK
                //before calling GetAttributesOf();

                ulAttrs=SFGAO_DISPLAYATTRMASK;

                lpsf->lpVtbl->GetAttributesOf(lpsf, 1, (LPCITEMIDLIST*)&lpi, &ulAttrs);
                lplvid->ulAttribs=ulAttrs;

                lpifqThisItem=ConcatPidls(lptvid->lpifq, lpi);

                lvi.iItem       = iCtr++;
                lvi.iSubItem    = 0;
                lvi.pszText     = LPSTR_TEXTCALLBACK;
                lvi.cchTextMax  = MAX_PATH;
                uFlags          = SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON;

                lvi.iImage      = I_IMAGECALLBACK;

                lplvid->lpsfParent=lpsf;
                lpsf->lpVtbl->AddRef(lpsf);

                //Now, make a copy of the ITEMIDLIST
                lplvid->lpi=CopyITEMID( lpMalloc, lpi );

                lvi.lParam = (LPARAM)lplvid;

                // Add the item to the listview
                if (ListView_InsertItem( hwndLV, &lvi ) == -1 )
                   return FALSE;

                lpMalloc->lpVtbl->Free( lpMalloc, lpifqThisItem );
                lpifqThisItem=0;
                lpMalloc->lpVtbl->Free( lpMalloc, lpi );  //Finally, free the pidl that the shell gave us...
                lpi=0;
            }
        }

    }

Done:

    // Hourglass off!

    ReleaseCapture();
    SetCursor(LoadCursor(NULL, IDC_ARROW));

    if (lpe)
			lpe->lpVtbl->Release(lpe);

    //The following 2 if statements will only be TRUE if we got here on an
    //error condition from the "goto" statement.  Otherwise, we free this memory
    //at the end of the while loop above.
    if (lpi && lpMalloc)           lpMalloc->lpVtbl->Free(lpMalloc, lpi);
    if (lpifqThisItem && lpMalloc) lpMalloc->lpVtbl->Free(lpMalloc, lpifqThisItem);

    if (lpMalloc) lpMalloc->lpVtbl->Release(lpMalloc);

    return TRUE;
}


//
//  FUNCTION: CreateListView(HWND)
//
//  PURPOSE: Create the listview control
//
//  PARAMETERS:
//
//    HWND - Handle to the parent window
//
//  RETURN VALUE:
//
//    hwndLV - Handle to listview control if successful
//    NULL   - if creation of listview control fails
//
//  COMMENTS:
//

HWND WINAPI CreateListView(HWND hwndParent)
{
    HWND hwndLV;
    RECT rc;

    // Force the common controls DLL to be loaded.

    InitCommonControls();

    // Create the control.

    GetClientRect(hwndParent, &rc);

    hwndLV = CreateWindowEx(WS_EX_CLIENTEDGE,
    	                    WC_LISTVIEW, "",
                            WS_VISIBLE | WS_CHILD | WS_BORDER | LVS_EDITLABELS |
                            LVS_LIST | LVS_AUTOARRANGE | LVS_SHAREIMAGELISTS,
                            0, 0,
                            rc.right-rc.left, rc.bottom-rc.top,
                            hwndParent,
                            (HMENU)IDD_LISTVIEW,
                            hInst,
                            NULL);

    return hwndLV;
}


//
//  FUNCTION: SwitchView(HWND, DWORD)
//
//  PURPOSE: Change the visual view of the listview control
//
//  PARAMETERS:
//
//    HWND  - Handle to the listview control
//    DWORD - View style indicating new view
//
//  RETURN VALUE:
//
//    void
//
//  COMMENTS:
//

void SwitchView(HWND hwndLV, DWORD dwView)
{
    DWORD dwStyle = GetWindowLong(hwndLV, GWL_STYLE);

    if ((dwStyle & LVS_TYPEMASK) != dwView)
        SetWindowLong(hwndLV, GWL_STYLE, (dwStyle & ~LVS_TYPEMASK) | dwView);
}


//
//  FUNCTION: ListViewCompareProc(LPARAM, LPARAM, LPARAM)
//
//  PURPOSE: Callback routine for sorting list
//
//  PARAMETERS:
//
//    lparam1    -
//    lparam2    -
//    lparamSort -
//
//
//  RETURN VALUE:
//
//    Always returns 0 - Message handled
//
//  COMMENTS:
//

int CALLBACK ListViewCompareProc(LPARAM lparam1,
                                 LPARAM lparam2,
                                 LPARAM lparamSort)
{
    LPLVITEMDATA lplvid1=(LPLVITEMDATA)lparam1;
    LPLVITEMDATA lplvid2=(LPLVITEMDATA)lparam2;
    HRESULT   hr;

    hr = lplvid1->lpsfParent->lpVtbl->CompareIDs(lplvid1->lpsfParent,
                                                 0,
                                                 lplvid1->lpi,
                                                 lplvid2->lpi);

    if (FAILED(hr))
       return 0;

    return (short)SCODE_CODE(GetScode(hr));
}


//
//  FUNCTION: PopulateListView(HWND, HWND, LPTVITEMDATA)
//
//  PURPOSE: Fill the listview
//
//  PARAMETERS:
//
//    HWND  - Handle to the listview control
//    DWORD - View style indicating new view
//
//  RETURN VALUE:
//
//    void
//
//  COMMENTS:
//


BOOL PopulateListView(HWND hwndLV,
                      LPTVITEMDATA lptvid,
                      LPSHELLFOLDER lpsf)
{
	int i = sizeof(TVITEMDATA);

    ListView_DeleteAllItems(hwndLV);


    if (!InitListViewImageLists(hwndLV) ||
        !InitListViewItems(hwndLV, lptvid, lpsf))
    {
        return FALSE;
    }

    ListView_SortItems(hwndLV,
                       ListViewCompareProc,
                       0);
}


